#!/usr/bin/perl
#
# redirector.pl
# version 1.21
# 
# This is the main redirector script that Squid will use to
# block those pesky banners.  Add a line like this to squid.conf:
# redirect_program /path/to/redirector.pl
#
# We now use a configuration file for settings; see the README
# file for more information.
#

my $config='bannerfilter.conf';
my @search=('/usr/local/squid/bannerfilter-1.21',
            '/usr/local/squid/etc',
            '/usr/local/etc',
            '/usr/etc',
            '/etc/squid',
            '/etc');

# Handle "-c /path/to/bannerfilter.conf"
while(defined($_=shift @ARGV)) {
  if($_ eq '-c') {
    $config=shift @ARGV;
    if($config=~/^(.*)\/(.*)$/) {
      @search=($1);
      $config=$2;
    }
    GiveHelp() unless($config);
  } else {
    GiveHelp();
  }
}

# Read the configuration file
ReadConf($config,@search);

# Load the data files
ReadData();

# Trap for SIGHUP, and set a flag if received
$SIG{'HUP'}=sub {$reload=1};

# Don't try to buffer output
$|=1;

# Loop infinitely, getting URLs from Squid
while (<>) {
  my @input=split;
  my $url=$input[0];
  my $newurl='';
  my $log='';

  # If SIGHUP was received
  ReadConf($config,@search) if($reload);
  ReadData() if($reload);

  my $WWW=$conf{'www'};
  $WWW.='/' unless($WWW=~/\/$/);
  if($url=~/^http:\/\/bannerfilter.internal\/(.*)$/) {
    $newurl=$WWW.$1;
    $log='http://bannerfilter.internal/';
  }
  unless($newurl) { 
    foreach my $x (@exceptions) {
      if(($x=~/(.*)\*(.*)/ ? (index($url,$1)+1 && index($url,$2)+1) : index($url,$x)+1)) {
        $newurl=$url;
        $log=$x;
        last;
      }
    }
  }
  unless($newurl) {
    foreach my $z (@frames) {
      my($x,$r)=split /\t/,$z;
      if(($x=~/(.*)\*(.*)/ ? (index($url,$1)+1 && index($url,$2)+1) : index($url,$x)+1)) {
        $newurl=$WWW.$r;
        $log=$x;
        last;
      }
    }
  }
  unless($newurl) {
    foreach my $x (@popups) {
      if(($x=~/(.*)\*(.*)/ ? (index($url,$1)+1 && index($url,$2)+1) : index($url,$x)+1)) {
        $newurl=$WWW.'popup.html';
        $log=$x;
        last;
      }
    }
  }
  unless($newurl) {
    foreach my $x (@banners) {
      if(($x=~/(.*)\*(.*)/ ? (index($url,$1)+1 && index($url,$2)+1) : index($url,$x)+1)) {
        $newurl=$WWW.$conf{'bannergif'};
        $log=$x;
        last;
      }
    }
  }
  LogBanner($url,$log) if($log);
  $newurl=$url unless($newurl);
  print("$newurl\n");
}

sub ReadData {
  (@banners,@popups,@frames,@exceptions)=((),(),(),());
  my $DATA=$conf{'data'};
  $DATA.='/' unless($DATA=~/\/$/);

  # Read the list of banners
  foreach my $file ('banners.data','banners.local.data') {
    open(FILE,$DATA.$file) || die("Can't open $file");
    while(<FILE>) {
      chop;
      push(@banners,$_) if($_ && !(/^#/));
    }
    close(FILE);
  }

  # Read the list of popup windows
  foreach my $file ('popups.data','popups.local.data') {
    open(FILE,$DATA.$file) || die("Can't open $file");
    while(<FILE>) {
      chop;
      push(@popups,$_) if($_ && !(/^#/));
    }
    close(FILE);
  }

  # Read the list of frame banners
  foreach my $file ('frames.data','frames.local.data') {
    open(FILE,$DATA.$file) || die("Can't open $file");
    while(<FILE>) {
      chop;
      push(@frames,"$1\t$2") if($_ && !(/^#/) && /^(\S+)\s+(\S+)$/);
    }
    close(FILE);
  }

  # Read the list of exceptions
  foreach my $file ('exceptions.data','exceptions.local.data') {
    open(FILE,$DATA.$file) || die("Can't open $file");
    while(<FILE>) {
      chop;
      push(@exceptions,$_) if($_ && !(/^#/));
    }
    close(FILE);
  }

  # Clear the flag so we don't do this again next time
  $reload=0;
}

sub LogBanner {
  if($conf{'log'}) {
    open(LOG,">>$conf{'log'}");
    print(LOG "$_[0] matches $_[1]\n");
    close LOG;
  }
}

sub ReadConf {
  # Arguments: filename, list of paths to search

  # Read the configuration file, and store values in %hash.
  # Format is "label = value" on each line; the label is case-insensitive
  # and stored in lower-case.  Any whitespace is allowed but not required
  # before and after the equals sign.  Comments and blank lines are
  # ignored.  Extra lines will be read but will be ignored by the script.

  %conf=();
  my $filename=shift(@_);
  my $path='';
  foreach my $x (@_) {
    if(-e "$x/$filename") {
      $path=$x;
      last;
    }
  }
  die "Can't find $filename" unless($path);
  open(CONF,"$path/$filename") || die "Can't open $filename";
  while(<CONF>) {
    chomp;
    next unless($_);
    next if(/^#/);
    if(/^([a-z]+)\s*?=\s*(.*)$/i) {
      $conf{lc($1)}=$2;
    } else {
      die "Syntax error in $filename";
    }
  }
  close(CONF);
}

sub GiveHelp {
  print("Syntax: $0 [-c /path/to/bannerfilter.conf]\n");
  exit;
}
