#!/bin/sh
#
# update.sh
# BannerFilter, http://phroggy.com/bannerfilter/
#
# This script downloads new updated versions of the four data files
# used by redirector.pl, overwrites the data files in the current
# directory, and makes redirector.pl reload the files.  This script
# requires lynx, wget, curl, or another HTTP downloader to be installed
# and working (see below).
#
# Because new versions of these data files may require BannerFilter to
# be upgraded, this script also checks for the current version of
# BannerFilter and notifies you when an update is available.  You will
# only be notified of required updates; if your old version of
# BannerFilter still works fine with the latest data files, you will
# not be notified if an update is available.
#
# If running as a non-root user, be sure you have write access to the
# current directory.  If you're not running as root or the user Squid is 
# running as, or if you don't have killall, comment out the last line.

# Choose your HTTP downloader:
CMD="lynx -source"
# CMD="wget -q -O -"
# CMD="curl -s"

# Earliest version of BannerFilter compatible with your data files:
OLD="1.2"

# Check to see if an upgrade is required
$CMD http://phroggy.com/bannerfilter/version.txt>version.tmp \
&& grep -i "^version " version.tmp>/dev/null \
|| { echo "Error retrieving version.txt"; exit; }

NEW=`head -1 version.tmp|cut -d " " -f2`
rm version.tmp

if [ "$OLD" != "$NEW" ]; then
  echo "The contents of BannerFilter's data files have changed, and you must"
  echo "upgrade to a new version of BannerFilter to be compatible with the"
  echo "changes Please upgrade to at least version $NEW at"
  echo "http://phroggy.com/bannerfilter/, and be sure to read the CHANGES."
  exit
fi

# Download all four data files

$CMD http://phroggy.com/bannerfilter/banners.data>banners.data.tmp \
&& grep "^# banners.data$" banners.data.tmp>/dev/null \
|| { echo "Error retrieving banners.data"; exit; }

$CMD http://phroggy.com/bannerfilter/popups.data>popups.data.tmp \
&& grep "^# popups.data$" popups.data.tmp>/dev/null \
|| { echo "Error retrieving popups.data"; exit; }

$CMD http://phroggy.com/bannerfilter/frames.data>frames.data.tmp \
&& grep "^# frames.data$" frames.data.tmp>/dev/null \
|| { echo "Error retrieving frames.data"; exit; }

$CMD http://phroggy.com/bannerfilter/exceptions.data>exceptions.data.tmp \
&& grep "^# exceptions.data$" exceptions.data.tmp>/dev/null \
|| { echo "Error retrieving exceptions.data"; exit; }

# Overwrite the data files with what was downloaded
mv banners.data.tmp banners.data
mv popups.data.tmp popups.data
mv frames.data.tmp frames.data
mv exceptions.data.tmp exceptions.data

# Send all instances of redirector.pl a signal to reload the data files
killall -HUP redirector.pl
